import { Mapper } from '@automapper/core';
import { InjectMapper } from '@automapper/nestjs';
import { QueryHandler } from '@nestjs/cqrs';
import { Messager } from '../../adapters/secondaries/messager';
import { MatchQuery } from '../../queries/match.query';
import { AdRepository } from '../../adapters/secondaries/ad.repository';
import { Match } from '../entities/match';
import { ICollection } from 'src/modules/database/src/interfaces/collection.interface';

@QueryHandler(MatchQuery)
export class MatchUseCase {
  constructor(
    private readonly _repository: AdRepository,
    private readonly _messager: Messager,
    @InjectMapper() private readonly _mapper: Mapper,
  ) {}

  async execute(matchQuery: MatchQuery): Promise<ICollection<Match>> {
    try {
      const paths = [];
      for (let i = 0; i < 2000; i++) {
        paths.push({
          key: 'route' + i,
          points: [
            {
              lat: 48.110899,
              lon: -1.68365,
            },
            {
              lat: 48.131105,
              lon: -1.690067,
            },
            {
              lat: 48.56516,
              lon: -1.923553,
            },
            {
              lat: 48.622813,
              lon: -1.997177,
            },
            {
              lat: 48.67846,
              lon: -1.8554,
            },
          ],
        });
      }
      const routes = await matchQuery.algorithmSettings.georouter.route(paths, {
        withDistance: true,
        withPoints: true,
        withTime: false,
      });
      const match = new Match();
      match.uuid = 'e23f9725-2c19-49a0-9ef6-17d8b9a5ec85';
      // this._messager.publish('matcher.match', 'match !');
      return {
        data: [match],
        total: 1,
      };
    } catch (error) {
      // this._messager.publish(
      //   'logging.matcher.match.crit',
      //   JSON.stringify({
      //     matchQuery,
      //     error,
      //   }),
      // );
      throw error;
    }
  }
}
