import { Test, TestingModule } from '@nestjs/testing';
import { HttpService } from '@nestjs/axios';
import { NamedRoute } from '../../domain/entities/named-route';
import { GeorouterCreator } from '../../adapters/secondaries/georouter-creator';
import { IGeorouter } from '../../domain/interfaces/georouter.interface';
import { of } from 'rxjs';
import { AxiosError } from 'axios';

const mockHttpService = {
  get: jest
    .fn()
    .mockImplementationOnce(() => {
      throw new AxiosError('Axios error !');
    })
    .mockImplementation(() => {
      return of({
        status: 200,
        data: [new NamedRoute()],
      });
    }),
};

describe('Graphhopper Georouter', () => {
  let georouterCreator: GeorouterCreator;
  let graphhopperGeorouter: IGeorouter;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [],
      providers: [
        GeorouterCreator,
        {
          provide: HttpService,
          useValue: mockHttpService,
        },
      ],
    }).compile();

    georouterCreator = module.get<GeorouterCreator>(GeorouterCreator);
    graphhopperGeorouter = georouterCreator.create(
      'graphhopper',
      'http://localhost',
    );
  });

  it('should be defined', () => {
    expect(graphhopperGeorouter).toBeDefined();
  });

  describe('route function', () => {
    it('should fail on axios error', async () => {
      await expect(
        graphhopperGeorouter.route(
          [
            {
              key: 'route1',
              points: [
                {
                  lat: 49.440041,
                  lon: 1.093912,
                },
                {
                  lat: 50.630992,
                  lon: 3.045432,
                },
              ],
            },
          ],
          {
            withDistance: false,
            withPoints: false,
            withTime: false,
          },
        ),
      ).rejects.toBeInstanceOf(Error);
    });
    it('should create one route with all settings to false', async () => {
      const routes = await graphhopperGeorouter.route(
        [
          {
            key: 'route1',
            points: [
              {
                lat: 49.440041,
                lon: 1.093912,
              },
              {
                lat: 50.630992,
                lon: 3.045432,
              },
            ],
          },
        ],
        {
          withDistance: false,
          withPoints: false,
          withTime: false,
        },
      );
      expect(routes).toHaveLength(1);
    });
    it('should create 2 routes with distance, points and time', async () => {
      const routes = await graphhopperGeorouter.route(
        [
          {
            key: 'route1',
            points: [
              {
                lat: 49.440041,
                lon: 1.093912,
              },
              {
                lat: 50.630992,
                lon: 3.045432,
              },
            ],
          },
          {
            key: 'route2',
            points: [
              {
                lat: 49.440041,
                lon: 1.093912,
              },
              {
                lat: 50.630992,
                lon: 3.045432,
              },
            ],
          },
        ],
        {
          withDistance: true,
          withPoints: true,
          withTime: true,
        },
      );
      expect(routes).toHaveLength(2);
    });
  });
});
